import unittest

import nlp2go


class TestMain(unittest.TestCase):

    def test_parser(self):
        input_arg, model_arg = nlp2go.parse_args(['--model', 'voidful/albert_chinese_tiny', '--task', 'clas'])
        print(input_arg, model_arg)



import inspect

import nlp2
import tfkit
from transformers import pipeline, pipelines, BertTokenizer, cached_path, AutoTokenizer

from nlp2go.modelhub import MODELMAP
from nlp2go.parser import Parser


class Model:

    def __init__(self, model_path, panel=False, **param):
        self.model = None
        self.enable_panel = panel
        if nlp2.is_file_exist(model_path) or "tfkit_" in model_path:  # tfkit models
            self.lib = 'tfkit'
            self.model, self.predict_parameter, self.model_task = self.load_tfkit_model(model_path, **param)
            self.predict_func = self.model.predict
        else:  # huggingface's transfromers model - local model saved in dir, online model name without tfkit tag
            self.lib = 'hf'
            self.model, self.predict_parameter, self.model_task = self.load_huggingface_model(model_path, **param)
            self.predict_func = self.model
        self.parser = Parser(self.model_task, self.predict_func, self.model.tokenizer)
        predict_parameter, _ = nlp2.function_sep_suit_arg(self.parser.get_input_parser(), param)
        self.predict_parameter.update(predict_parameter)
        print("loaded model predict_parameter", predict_parameter)

    def load_huggingface_model(self, model_path, **param):
        supported_type = list(pipelines.SUPPORTED_TASKS.keys())
        if 'task' not in param or param['task'] not in supported_type:
            panel = nlp2.Panel()
            panel.add_element('task', supported_type, "Select model task: ", default={})
            model_task = panel.get_result_dict()['task']
        else:
            model_task = param['task']

        param['model'] = model_path
        param['tokenizer'] = BertTokenizer.from_pretrained(model_path) if 'voidful/albert_chinese' in model_path \
            else AutoTokenizer.from_pretrained(model_path)
        pipeline_param, _ = nlp2.function_sep_suit_arg(pipeline, param)
        nlp = pipeline(**pipeline_param)
        predict_parameter, _ = nlp2.function_sep_suit_arg(nlp, param)
        return nlp, predict_parameter, model_task

    def load_tfkit_model(self, model_path, **param):
        model_task = param['task'] if 'task' in param else None
        model_path = MODELMAP[model_path] if model_path in MODELMAP else model_path
        model, model_task, model_class, model_info = tfkit.utility.model_loader.load_trained_model(cached_path(model_path),
                                                                                       tag=model_task)
        predict_parameter, _ = nlp2.function_sep_suit_arg(model.predict, param)
        model.eval()
        return model, predict_parameter, model_task

    def predict(self, pred_json=None, **argument):
        pred_param = self.predict_parameter.copy()
        pred_param.update(argument)
        if pred_json:
            if isinstance(pred_json, str):
                pred_param.update({'input': pred_json})
            else:
                pred_param.update(pred_json)
        input_argument = self.parser.input_parser(pred_param, enable_arg_panel=self.enable_panel)
        try:
            if isinstance(input_argument, tuple):
                pred = self.predict_func(input_argument[0], **input_argument[1])
            else:
                pred = self.predict_func(**input_argument)
            pred = self.parser.output_parser(pred)
            return pred
        except Exception as e:
            return {
                "input": input_argument,
                "error": str(e)
            }



import unittest

from transformers import pipelines

import nlp2go


class TestModel(unittest.TestCase):

    def test_load_huggingface_model(self):
        nlp, predict_parameter, model_task = nlp2go.Model.load_huggingface_model(self, 'voidful/albert_chinese_tiny',
                                                                                 task='feature-extraction')
        print(nlp, predict_parameter, model_task)
        self.assertTrue(len(predict_parameter) == 0)

        nlp, predict_parameter, model_task = nlp2go.Model.load_huggingface_model(self, 'voidful/albert_chinese_tiny',
                                                                                 task="fill-mask")
        print(nlp, predict_parameter, model_task)
        self.assertTrue(len(predict_parameter) == 0)

    def test_predict_hf(self):
        supported_type = list(pipelines.SUPPORTED_TASKS.keys())
        ignoree_type = ['table-question-answering', 'summarization', 'translation', 'text2text-generation',
                        'text-generation', 'conversational']
        for task in supported_type:
            print(task)
            if task not in ignoree_type:
                model = nlp2go.Model('voidful/albert_chinese_tiny', task=task)
            else:
                continue
            result = model.predict(input="I [MASK] Fine.")
            print(result)
            self.assertIsInstance(result, dict)
            result = model.predict("I [MASK] Fine.")
            print(result)
            self.assertIsInstance(result, dict)
            result = model.predict({"contexta": {"input": "I [MASK] ok.", "order": 0},
                                    "contextb": {"input": "I [MASK] Fine.", "field": "input", "order": 1}})
            print(result)
            self.assertIsInstance(result, dict)

    def test_predict_hf_generate_with_parama(self):
        model = nlp2go.Model('sshleifer/tiny-gpt2', task="text-generation")
        result_dict = model.predict("I [MASK] Fine.", num_return_sequences=3)
        print(result_dict)
        self.assertEqual(len(result_dict['result']), 3)

    def test_predict_hf_qa(self):
        model = nlp2go.Model("sshleifer/tiny-distilbert-base-cased-distilled-squad", task="question-answering")
        result_dict = model.predict(question="How old are you.", context="i am 10 years old")
        print(result_dict)

    def test_predict_tfkit(self):
        # tfkit pipeline
        from nlp2go.modelhub import MODELMAP
        for k in MODELMAP.keys():
            model = nlp2go.Model(k)
            if "mrc" not in k:
                result_dict = model.predict(input="今季新番有咩睇")
                print(result_dict)
                self.assertIsInstance(result_dict, dict)
                result_dict = model.predict("今季新番有咩睇")
                print(result_dict)
                self.assertIsInstance(result_dict, dict)
                result_dict = model.predict({"contexta": {"input": "今季新番有咩睇", "order": 0},
                                             "contextb": {"input": "冇啊", "field": "input", "order": 1}})
                self.assertIsInstance(result_dict, dict)
            else:
                result_dict = model.predict(passage="今季冇新番", question="今季新番有咩睇", topk=10)
                print(result_dict)
                self.assertIsInstance(result_dict, dict)
                result_dict = model.predict({"contexta": {"input": "今季新番有咩睇", "field": "question"},
                                             "contextb": {"input": "今季冇新番", "field": "passage"}})
                print(result_dict)
                self.assertIsInstance(result_dict, dict)

    def test_predict_tfkit_with_parama(self):
        model = nlp2go.Model("tfkit_zh_dream_small")
        result_dict = model.predict("今季新番有咩睇", decodenum=3)
        print(result_dict)
        self.assertTrue(len(result_dict['result']) == 3)



import argparse
from collections import defaultdict
import json

from nlp2go.cli import Cli
from nlp2go.model import Model
from nlp2go.server import Server

import os

os.environ["PYTHONIOENCODING"] = "utf-8"


def parse_args(args):
    parser = argparse.ArgumentParser()
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('--json', type=str)
    group.add_argument('--model', type=str)
    parser.add_argument('--panel', action='store_true', help='enable panel to input argument')
    # interface
    parser.add_argument('--api_path', type=str, default="model", help='api path to serve the demo')
    parser.add_argument('--api_port', type=int, default=3000, help='port to serve the demo on')
    parser.add_argument('--cli', action='store_true', help='commandline mode')

    input_arg, others_arg = parser.parse_known_args(args)
    input_arg = {k: v for k, v in vars(input_arg).items() if v is not None}
    others_arg = {k.replace("--", ""): v for k, v in zip(others_arg[:-1:2], others_arg[1::2])}
    return input_arg, others_arg


def main(arg=None):
    """
    loading argument and starting an model interface
    """
    global model_dict
    model_dict = defaultdict(dict)
    args, others_arg = parse_args(arg)
    loaded_model = {}
    if args.get('model', None):
        loaded_model[args.get('api_path')] = Model(args.get('model'), args.get('panel', False), **others_arg)
    else:
        with open(args.get('json'), 'r', encoding='utf8') as reader:
            model_dict = json.loads(reader.read())
        for path, d in model_dict.items():
            model = Model(model_dict[path]['model'], args.get('panel', False), **model_dict[path])
            loaded_model[path] = model
            model_dict[path].pop('model')

    if args.get('cli', False):
        cli = Cli()
        cli.start(loaded_model)
    else:
        server = Server()
        server.start(loaded_model, args.get('api_port'), model_dict)


if __name__ == "__main__":
    main()



from collections import OrderedDict, defaultdict
import nlp2
import tfkit


class Parser:
    def __init__(self, model_type, model, tokenizer):
        self.model = model
        self.tokenizer = tokenizer
        im = self.INPUT_PARSER_MAPPING
        om = self.OUTPUT_PARSER_MAPPING
        self._input_parser = im[model_type] if model_type in im else im['general']
        self._output_parser = om[model_type] if model_type in om else om['general']

    def input_parser(self, argument={}, enable_arg_panel=False):
        argument = nlp2.function_argument_panel(self._input_parser, inputted_arg=argument,
                                                disable_input_panel=(not enable_arg_panel))
        # error input check
        miss = nlp2.function_check_missing_arg(self._input_parser, list(argument.keys()))
        if len(miss) > 0:
            return {"miss": miss, "all": nlp2.function_get_all_arg(self._input_parser)}
        else:
            return self._input_parser(self, **argument)

    def get_input_parser(self):
        return self._input_parser

    def _input_general_parser(self, input="", **pred_arg):
        input_dict = defaultdict(lambda: OrderedDict())
        for k, v in list(pred_arg.items()):
            if isinstance(v, dict):
                input_dict[v.get('field', 'input')][v.get('order', 0)] = v.get('input')
                pred_arg.pop(k)
        for k, v in input_dict.items():
            if k not in pred_arg:
                pred_arg[k] = ""
            pred_arg[k] += tfkit.utility.tok.tok_sep(self.tokenizer).join(v.values())
        if "input" in pred_arg:
            input = pred_arg["input"]
            pred_arg.pop("input")
        return input, pred_arg

    def _input_hf_fillmask_parser(self, input="", targets=None, top_k=5, **pred_arg):
        param = {"targets": targets, "top_k": top_k}
        param.update(pred_arg)
        input, param = self._input_general_parser(input, **param)
        return input, param

    def _input_hf_generate_parser(self, input="", max_length=20, min_length=10, num_return_sequences=1, num_beams=1,
                                  **pred_arg):
        param = {"max_length": max_length, "min_length": min_length, "num_return_sequences": num_return_sequences,
                 "num_beams": num_beams}
        param.update(pred_arg)
        input, param = self._input_general_parser(**param)
        return input, param

    def _input_hf_qa_parser(self, passage="", question="", **pred_arg):
        param = {"context": passage, "question": question}
        param.update(pred_arg)
        input, param = self._input_general_parser(**param)
        return input, param

    def _input_tfkit_qa_parser(self, passage="", question="", **pred_arg):
        _, param = self._input_general_parser(**pred_arg)
        if "passage" in param:
            passage = param["passage"]
            param.pop("passage", None)
        if "question" in param:
            question = param["question"]
            param.pop("question", None)
        sep = tfkit.utility.tok.tok_sep(self.tokenizer)
        param['input'] = passage + sep + question
        return param

    def output_parser(self, outputs):
        return self._output_parser(self, outputs)

    def _outout_tfkit_parser(self, result_triple):
        result, detail = result_triple
        return {
            'result': result
        }

    def _outout_hf_parser(self, result):
        return {
            'result': result
        }

    OUTPUT_PARSER_MAPPING = OrderedDict(
        [
            # Huggingface's model
            ("question-answering", _outout_hf_parser,),
            ("feature-extraction", _outout_hf_parser,),
            ("sentiment-analysis", _outout_hf_parser,),
            ("ner", _outout_hf_parser,),
            ("fill-mask", _outout_hf_parser,),
            ("summarization", _outout_hf_parser,),
            ("text2text-generation", _outout_hf_parser,),
            ("translation_xx_to_yy", _outout_hf_parser,),
            ("zero-shot-classification", _outout_hf_parser,),
            ("text-generation", _outout_hf_parser,),
            ("conversational", _outout_hf_parser,),
            # TFKIT model
            ("general", _outout_tfkit_parser,),
        ]
    )

    INPUT_PARSER_MAPPING = OrderedDict(
        [
            # Huggingface's model
            ("feature-extraction", _input_general_parser,),
            ("sentiment-analysis", _input_general_parser,),
            ("ner", _input_general_parser,),
            ("question-answering", _input_hf_qa_parser,),
            ("fill-mask", _input_hf_fillmask_parser,),
            ("summarization", _input_hf_generate_parser,),
            ("text2text-generation", _input_hf_generate_parser,),
            ("translation_xx_to_yy", _input_general_parser,),
            ("zero-shot-classification", _input_general_parser,),
            ("text-generation", _input_hf_generate_parser,),
            ("conversational", _input_hf_generate_parser,),
            # TFKIT model
            ("qa", _input_tfkit_qa_parser,),
            ("general", _input_general_parser,),
        ]
    )



import argparse
from collections import defaultdict

import json

from nlp2go import parse_args
from nlp2go.model import Model


def main(arg=None):
    global model_dict
    model_dict = defaultdict(dict)
    args, others_arg = parse_args(arg)

    if args.get('model', None):
        Model(args.get('model'), args.get('panel', False), **others_arg)
    else:
        with open(args.get('json'), 'r', encoding='utf8') as reader:
            model_dict = json.loads(reader.read())
        for path, d in model_dict.items():
            Model(model_dict[path]['model'], args.get('panel', False), **model_dict[path])

    print("==================")
    print("finish pre loading")


if __name__ == "__main__":
    main()
